﻿// (c) Copyright Morten Nielsen.
// This source is subject to the Microsoft Public License (Ms-PL).
// Please see http://go.microsoft.com/fwlink/?LinkID=131993 for details.
// All other rights reserved.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;

namespace AccelerometerPlane
{
	public partial class MainPage : PhoneApplicationPage
	{
		Microsoft.Devices.Sensors.Accelerometer meter;
		private double lastX=0;
		private double lastY=0;
		private double lastZ=1;
		private const double smoothing = .35;

		public MainPage()
		{
			InitializeComponent();
			meter = new Microsoft.Devices.Sensors.Accelerometer();
			meter.ReadingChanged += meter_ReadingChanged;
		}
		protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
		{
			base.OnNavigatedTo(e);
			meter.Start();
		}

		protected override void OnNavigatingFrom(System.Windows.Navigation.NavigatingCancelEventArgs e)
		{
			meter.Stop();
			base.OnNavigatingFrom(e);
		}
		protected override void OnOrientationChanged(OrientationChangedEventArgs e)
		{
			base.OnOrientationChanged(e);
			UpdateRotation();
		}

		private void meter_ReadingChanged(object sender, Microsoft.Devices.Sensors.AccelerometerReadingEventArgs e)
		{
			Dispatcher.BeginInvoke(() =>
			{
				//Low-pass filter to smooth readings a bit:
				lastX = lastX + smoothing * (e.X - lastX);
				lastY = lastY + smoothing * (e.Y - lastY);
				lastZ = lastZ + smoothing * (e.Z - lastZ);
				UpdateRotation();
			});
		}

		private void UpdateRotation()
		{
			var offset = 0d;
			//Adjust for screen orientation
			if (Orientation == PageOrientation.LandscapeLeft) offset = .5;
			else if (Orientation == PageOrientation.LandscapeRight) offset = 1.5;				
			else if (Orientation == PageOrientation.PortraitUp) { /*Do nothing*/ }
			var x = (Math.Acos(lastZ) / Math.PI + 1) * 180;
			var z = (offset - Math.Atan2(lastX, lastY) / Math.PI + 1) * 180;
			if(!double.IsNaN(z)) proj.RotationZ = z;
			if(!double.IsNaN(x)) proj.RotationX = x;
		}
	}
}